import java.io.FileReader;
import java.io.IOException;

import common.Diagnostics;
import lexer.Lexer;
import lexer.tokens.StaticToken;
import lexer.tokens.Tag;
import lexer.tokens.Token;

public class Compiler {
	public static void main(String[] arguments) throws IOException {
		if (arguments.length != 1) {
			System.out.println("Usage: java Compiler <file>");
		}
		String inputFile = arguments[0];
		try (var reader = new FileReader(inputFile)) {
			var diagnostics = new Diagnostics();
			var lexer = new Lexer(reader, diagnostics);
			printTokens(lexer);
			if (failed(diagnostics)) {
				System.out.println("Exit after syntax errors");
				return;
			}
		}
	}

	private static void printTokens(Lexer lexer) {
		var token = lexer.next();
		while (!isEndToken(token)) {
			System.out.println(token);
			token = lexer.next();
		}
	}

	private static boolean isEndToken(Token token) {
		return token instanceof StaticToken && ((StaticToken)token).getTag() == Tag.END;
	}

	private static boolean failed(Diagnostics diagnostics) {
		if (diagnostics.hasErrors()) {
			for (String message : diagnostics.getErrors()) {
				System.out.println(message);
			}
			return true;
		}
		return false;
	}
}
